/**
 * Beschreibt kubische Polynome.
 * 
 * @author Owyn
 * @version 2008-05-23
 */
public class Polynom {
    
    double a;
    double b;
    double c;
    double d;
    
    /**
     * Erzeugt ein konstantes Polynom.
     * 
     * @param d f(x) = d
     */
    public Polynom(double d) {
        a = b = c = 0.0;
        this.d = d;
    }

    /**
     * Erzeugt ein lineares Polynom.
     * 
     * @param c f(x) = c*x ...
     * @param d ... + d
     */
    public Polynom(double c, double d) {
        a = b = 0.0;
        this.c = c;
        this.d = d;
    }

    /**
     * Erzeugt ein quadratisches Polynom.
     * 
     * @param b f(x) = b*x^2 ...
     * @param c ... + c*x ...
     * @param d ... + d
     */
    public Polynom(double b, double c, double d) {
        a = 0.0;
        this.b = b;
        this.c = c;
        this.d = d;
    }

    /**
     * Erzeugt ein kubisches Polynom.
     * 
     * @param a f(x) = a*x^3 ...
     * @param b ... + b*x^2 ...
     * @param c ... + c*x ...
     * @param d ... + d
     */
    public Polynom(double a, double b, double c, double d) {
        this.a = a;
        this.b = b;
        this.c = c;
        this.d = d;
    }
    
    /**
     * Addiert zwei Polynome.
     * 
     * @param p Zweites Polynom
     * @return Neues Polynom, das die Summe aus dem aktuellen Objekt und p darstellt
     */
    public Polynom add(Polynom p){
        return new Polynom(a + p.a, b + p.b, c + p.c, d + p.d);
    }
    
    /**
     * Wertet das Polynom zu einem Wert aus.
     * 
     * @param x Stelle, an der das POlynom ausgewertet werden soll.
     * @return f(x)
     */
    public double apply(double x) {
        return ((a*x + b)*x + c)*x + d; // Hier wird das Horner Schema angewendet
    }
    
    /**
     * Leitet das Polynom nach x ab.
     * 
     * @return Neues Polynom, das die Ableitung des aktuellen Objekts darstellt
     */
    public Polynom dx() {
        return new Polynom(0, 3 * a, 2 * b, c);
    }
    
    /**
     * Bildet das Integral des Polynoms.
     * 
     * @return Neues Polynom, das das Integral des aktuellen Objekts darstellt
     */
    public Polynom integral() {
        return new Polynom(b/3, c/2, d, 0);
    }
    
    /**
     * Gibt das Polynom auf dem Bildschirm aus.
     */
    public void print() {
        // Boolesche Variable zum Test, ob es sich
        // um den ersten auszugebenden Koeffizienten handelt.
        boolean firstCoeff = true;
        
        //Ausdrucken von a und x^3, falls nicht 0
        if (a != 0) {
            firstCoeff = false;
            //Ausdrucken von a falls auch nicht 1
            if (a != 1) System.out.print(a + "*");
            System.out.print("x^3 ");
        }
        
        if (b != 0) {
            if (firstCoeff) {
                firstCoeff = false;
                if (b < 0) System.out.print("-");
            } else {
                //Falls nicht erster Koeffizient, Additionszeichen einsetzen,
                System.out.print(b > 0?"+ ":"- ");
            }
            if (b != 1) System.out.print(Math.abs(b) + "*");
            System.out.print("x^2 ");
        }
        
        if (c != 0) {
            if (firstCoeff) {
                firstCoeff = false;
                if (c < 0) System.out.print("-");
            } else {
                System.out.print(c > 0?"+ ":"- ");
            }
            if (c != 1) System.out.print(Math.abs(c) + "*");
            System.out.print("x ");
        }
        
        if (d != 0) {
            if (firstCoeff) {
                firstCoeff = false;
                if (d < 0) System.out.print("-");
            } else {
                System.out.print(d > 0?"+ ":"- ");
            }
            System.out.print(Math.abs(d));
        }
        System.out.println();
    }
    
}
